<?php

namespace App\Http\Controllers\Admin\Base;

use App\Contracts\Repositories\Base\AdminRepository;
use App\Http\Controllers\Controller;
use App\Jobs\Base\ImportAdminJob;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Base\Admin;
use App\Repositories\Models\Base\Resource;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Repositories\Validators\Base\AdminValidator;

/**
 * Class AdminsController.
 *
 * @package namespace App\Http\Controllers;
 */
class AdminController extends Controller
{
    /**
     * @var AdminRepository
     */
    protected $repository;

    /**
     * @var AdminValidator
     */
    protected $validator;

    /**
     * AdminsController constructor.
     *
     * @param AdminRepository $repository
     * @param AdminValidator $validator
     */
    public function __construct(AdminRepository $repository, AdminValidator $validator)
    {
        parent::__construct();
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index()
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 创建用户
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        if ($msg = check_password($data['password'])) {
            return Response::fail($msg);
        }

        if ($data['password']) {
            $data['password'] = Hash::make($data['password']);
        }

        $role_ids = $data['role_ids'];
        unset($data['role_ids']);

        if (in_array(2, $role_ids) || in_array(3, $role_ids)) {
            $this->validate($request, [
                'department_id' => 'required|integer'
            ]);
        }

        try {
            $model = Admin::query()->create($data);

            if (count($role_ids)) {
                $model->syncRoles($role_ids);
            }

            return Response::success($model);
        } catch (\Exception $e) {
            return $this->error($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $patient = $this->repository->find($id);
        return Response::success($patient);
    }

    /**
     * 修改
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));

        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));

        if (isset($data['password']) && !is_null($data['password'])) {
            if ($msg = check_password($data['password'])) {
                return Response::fail($msg);
            }
            $data['password'] = Hash::make($data['password']);
        } else {
            unset($data['password']);
        }

        $model = Admin::query()->where('id', $id)->first();

        $role_ids = $data['role_ids'];

        if (in_array(2, $role_ids) || in_array(3, $role_ids)) {
            $this->validate($request, [
                'department_id' => 'required|integer'
            ]);
        }

        unset($data['role_ids']);
        if (count($role_ids)) {
            $model->syncRoles($role_ids);
        }

        $model->fill($data);
        $re = $model->save();
        if ($re) {
            return Response::success(null);
        }

        return $this->errorFail();
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $re = $this->repository->delete($id);
            if ($re) {
                return Response::success(null, T('successfully delete.'));
            }
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 用户授权
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function authorization(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'role_ids' => 'required|array',
        ]);

        $model = Admin::query()->find($request->get('id'));
        if (!$model) {
            return Response::fail('找不到该用户');
        }
        $role_ids = $request->get('role_ids');
        $model->syncRoles($role_ids);
        return Response::success(null);
    }

    /**
     * 选项列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     */
    public function selectOptions(Request $request)
    {
        $admins = $this->repository->select(['id', 'username', 'name'])->get();
        return Response::success($admins);
    }

    /**
     * 重置密码
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function changePassword(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'password' => 'required',
        ]);

        $id = $request->get('id');
        $password = $request->get('password');
        if ($msg = check_password($password)) {
            return Response::fail($msg);
        }

        $admin = Admin::query()->find($id);
        if (!$admin) {
            return Response::fail('找不到该用户');
        }

        $admin->password = Hash::make($password);
        $admin->save();
        return Response::success([]);
    }

    /**
     * 导入
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function import(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer'
        ]);

        $resource = Resource::query()->where('id', $request->get('id'))->first();
        if (!$resource) {
            return $this->errorFail();
        }
        $this->dispatch(new ImportAdminJob($resource['disk'], $resource['path']));
        return Response::success(null, '正在导入，请稍后');
    }
}
