<?php

namespace App\Http\Controllers\Admin\Lab;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\PermissionEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Lab\Maintain;
use App\Repositories\Models\School\LessonSchedule;
use App\Repositories\Presenters\Lab\MaintainPresenter;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Lab\MaintainRepository;
use App\Repositories\Validators\Lab\MaintainValidator;

/**
 * Class MaintainsController.
 *
 * @package namespace App\Http\Controllers\Lab;
 */
class MaintainsController extends Controller
{
    /**
     * @var MaintainRepository
     */
    protected $repository;

    /**
     * @var MaintainValidator
     */
    protected $validator;

    /**
     * MaintainsController constructor.
     *
     * @param MaintainRepository $repository
     * @param MaintainValidator $validator
     */
    public function __construct(MaintainRepository $repository, MaintainValidator $validator)
    {
        parent::__construct();
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 排班列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     */
    public function lists(Request $request)
    {
        $lists = $this->repository->all();
        return Response::success($lists);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        $data['term_id'] = self::$TERM_ID;
        $data['day'] = LessonSchedule::byTimeGetDate(self::$TERM_ID, $data['week'], $data['week_index']);
        $data['admin_id'] = login_admin_id();

        $maintain_admin_id = 0;
        if ($this->isCan(PermissionEnum::model_maintain_me, true)) {
            $maintain_admin_id = login_admin_id();
        }

        //检查是否有课
        $where = Arr::only($data, ['term_id', 'week', 'week_index', 'room_id', 'time_id']);
        $is_lesson = LessonSchedule::query()->where('status', ModelStatusEnum::OK)->where($where)->exists();
        if ($is_lesson) {
            return $this->error('该时间段有课，暂不能维护');
        }
        //检查是否已经存在
        $is_maintain = Maintain::query()->where('status', ModelStatusEnum::OK)->where($where)->exists();
        if ($is_maintain) {
            return $this->error('该时间段已申请维护');
        }

        if ($maintain_admin_id) {
            $data['maintain_admin_id'] = $maintain_admin_id;
        }

        try {
            $model = $this->repository->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->error($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $data = $this->repository->find($id);
        return Response::success($data);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->error($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        if ($this->isCan(PermissionEnum::model_maintain_me, true)) {
            if (Maintain::query()->where('id', $id)->where('maintain_admin_id', login_admin_id())->exists()) {
                return $this->error('您暂时没有权限操作');
            }
        }

        try {
            $this->repository->skipPresenter()->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        $this->validate($request, [
            'ids' => 'required|array',
        ]);

        $ids = $request->get('ids');
        if ($this->isCan(PermissionEnum::model_maintain_me, true)) {
            if (Maintain::query()->whereIn('id', $ids)->where('maintain_admin_id', login_admin_id())->exists()) {
                return $this->error('您暂时没有权限操作');
            }
        }

        try {
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 根据时间获取维护记录
     * @param Request $request
     * @return mixed
     * @throws \Illuminate\Validation\ValidationException
     */
    public function byTime(Request $request)
    {
        $this->validate($request, [
            'day' => 'required|date',
            'time_id' => 'required|integer',
            'room_id' => 'required|integer',
        ]);

        $day = $request->get('day');
        $time_id = $request->get('time_id');
        $room_id = $request->get('room_id');

        $model = Maintain::query()->where('status', ModelStatusEnum::OK)->where('day', $day)->where('time_id', $time_id)->where('room_id', $room_id)->where('term_id', self::$TERM_ID)->first();

        return (new MaintainPresenter())->present($model);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function batchStore(Request $request)
    {
        $this->validate($request, [
            'room_id' => 'required|integer',
            'maintain_data' => 'required|array',
        ]);

        $room_id = $request->get('room_id');
        $maintain_data = $request->get('maintain_data');

        $data = [];

        $maintain_admin_id = 0;
        if ($this->isCan(PermissionEnum::model_maintain_me, true)) {
            $maintain_admin_id = login_admin_id();
        }

        foreach ($maintain_data as $index => $maintain) {

            if (!Arr::has($maintain, ['week', 'week_index', 'time_id', 'maintain_admin_id', 'body'])) {
                return $this->error('第' . ($index + 1) . '条数据有误！');
            }

            $where = Arr::only($maintain, ['week', 'week_index', 'time_id']);
            $where['term_id'] = self::$TERM_ID;
            $where['room_id'] = $room_id;
            $is_lesson = LessonSchedule::query()->where('status', ModelStatusEnum::OK)->where($where)->exists();
            if ($is_lesson) {
                return $this->error('第' . ($index + 1) . '条数据维护时间有课，暂不能维护');
            }

            $is_maintain = Maintain::query()->where('status', ModelStatusEnum::OK)->where($where)->exists();
            if ($is_maintain) {
                return $this->error('第' . ($index + 1) . '条数据维护时间内，已存在维护申请');
            }

            $now = Carbon::now()->toDateTimeString();
            $data[] = [
                'term_id' => self::$TERM_ID,
                'room_id' => $room_id,
                'week' => $maintain['week'],
                'week_index' => $maintain['week_index'],
                'maintain_admin_id' => $maintain_admin_id ? $maintain_admin_id : $maintain['maintain_admin_id'],
                'time_id' => $maintain['time_id'],
                'admin_id' => login_admin_id(),
                'body' => $maintain['body'],
                'day' => LessonSchedule::byTimeGetDate(self::$TERM_ID, $maintain['week'], $maintain['week_index']),
                'status' => ModelStatusEnum::OK,
                'created_at' => $now,
                'updated_at' => $now
            ];
        }

        foreach ($data as $d) {
            Maintain::query()->create($d);
        }

        return Response::success([]);
    }

    /**
     * 暂停
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function pause(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'type' => 'sometimes|nullable|integer',
        ]);
        $id = $request->get('id');
        if ($this->isCan(PermissionEnum::model_maintain_me, true)) {
            if (Maintain::query()->where('id', $id)->where('maintain_admin_id', login_admin_id())->exists()) {
                return $this->error('您暂时没有权限操作');
            }
        }

        $status = ModelStatusEnum::PAUSE;
        if ($request->get('type')) {
            $status = ModelStatusEnum::OK;
        }
        $model = Maintain::query()->where('id', $id)->first();
        $model->status = $status;
        $re = $model->save();
        if ($re) {
            return Response::success([]);
        }
        return $this->error('操作失败');
    }
}
