<?php

namespace App\Http\Controllers\Admin\News;

use App\Http\Controllers\Controller;
use App\Http\Resources\News\MessageResource;
use App\Jobs\News\SendMessageJob;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\News\SendStatusEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Base\Admin;
use App\Repositories\Models\News\Message;
use App\Repositories\Models\News\Notice;
use App\Repositories\Models\News\UserMessage;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\News\MessageRepository;
use App\Repositories\Validators\News\MessageValidator;

/**
 * Class MessagesController.
 *
 * @package namespace App\Http\Controllers\News;
 */
class MessagesController extends Controller
{
    /**
     * @var MessageRepository
     */
    protected $repository;

    /**
     * @var MessageValidator
     */
    protected $validator;

    /**
     * MessagesController constructor.
     *
     * @param MessageRepository $repository
     * @param MessageValidator $validator
     */
    public function __construct(MessageRepository $repository, MessageValidator $validator)
    {
        parent::__construct();
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $lists = $this->repository->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

        if ($data['type'] == 1) {
            $this->validate($request, [
                'message' => 'required'
            ]);
        }

        if ($data['type'] == 2) {
            $this->validate($request, [
                'resource_type' => 'required',
                'resource_id' => 'required'
            ]);
            $data['resource_type'] = Notice::class;
        }

        if (empty($data['send_time'])) {
            $data['send_time'] = Carbon::now()->toDateTimeString();
        }
        try {
            $data['admin_id'] = login_admin_id();
            $data['status'] = SendStatusEnum::wait;
            $model = $this->repository->create($data);

            $this->dispatch(new SendMessageJob($model['id']));

            return Response::success($model);
        } catch (\Exception $e) {
            return $this->error($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $data = $this->repository->find($id);
        return Response::success($data);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
        $model = Message::query()->find($id);
        if (!$model) {
            return $this->error('找不到该记录');
        }

        if ($data['type'] == 1) {
            $this->validate($request, [
                'message' => 'required'
            ]);
        }

        if ($data['type'] == 2) {
            $this->validate($request, [
                'resource_type' => 'required',
                'resource_id' => 'required'
            ]);
            $data['resource_type'] = Notice::class;
        }

        try {
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->error($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        try {
            $this->repository->skipPresenter()->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function selectOptions(Request $request)
    {
        $data = $this->repository->skipPresenter()->all(['name', 'room_no', 'id']);
        return Response::success($data);
    }

    /**
     * 暂停
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function pause(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
            'type' => 'sometimes|nullable|integer',
        ]);
        $status = ModelStatusEnum::PAUSE;
        if ($request->get('type')) {
            $status = ModelStatusEnum::OK;
        }

        $re = Message::query()->where('id', $request->get('id'))->update(['status' => $status]);
        if ($re) {
            return Response::success([]);
        }
        return $this->error('操作失败');
    }

    /**
     * 用户列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     */
    public function users(Request $request)
    {
        $type = $request->get('type', false);
        $role_name = $request->get('role_name', false);
        $account = $request->get('account', false);
        $name = $request->get('name', false);
        $department_id = $request->get('department_id', false);
        $department_name = $request->get('department_name', false);

        $exclude_role_name = $request->get('exclude_role_name', false);
        $exclude_account = $request->get('exclude_account', false);
        $exclude_name = $request->get('exclude_name', false);
        $exclude_department_id = $request->get('exclude_department_id', false);
        $exclude_department_name = $request->get('exclude_department_name', false);

        $exclude_user_ids = [];
        if ($exclude_role_name || $exclude_account || $exclude_department_name || $exclude_department_id || $exclude_name) {
            $exclude_model = Admin::query();
            if ($exclude_role_name) {
                $exclude_model->whereHas('roles', function ($query) use ($exclude_role_name) {
                    return $query->where('name', '=', $exclude_role_name);
                });
            }

            if ($exclude_account) {
                $exclude_model->where('account', 'like', "%{$exclude_account}%");
            }

            if ($exclude_name) {
                $exclude_model->where('name', 'like', "%{$exclude_name}%");
            }

            if ($exclude_department_id) {
                $exclude_model->where('name', '=', $exclude_department_id);
            }

            if ($department_name) {
                $exclude_model->whereHas('department', function ($query) use ($exclude_department_name) {
                    return $query->where('name', 'like', "%{$exclude_department_name}%");
                });
            }

            $exclude_user_ids = $exclude_model->pluck('id')->toArray();
        }

        $model = Admin::query()->with(['department'])->select(['id', 'username', 'name', 'mobile', 'email', 'department_id']);

        if (count($exclude_user_ids)) {
            $model->whereNotIn('id', $exclude_user_ids);
        }
//
        if ($role_name) {
            $model->whereHas('roles', function ($query) use ($role_name) {
                return $query->where('name', '=', $role_name);
            });
        }

        if ($account) {
            $model->where('account', 'like', "%{$account}%");
        }

        if ($name) {
            $model->where('name', 'like', "%{$name}%");
        }

        if ($department_id) {
            $model->where('name', '=', $department_id);
        }

        if ($department_name) {
            $model->whereHas('department', function ($query) use ($department_name) {
                return $query->where('name', 'like', "%{$department_name}%");
            });
        }

        if ($type) {
            $users = $model->pluck('id');
        } else {
            $users = $model->paginate();
        }

        return Response::success($users);
    }

    /**
     * 我的消息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     */
    public function userMessages(Request $request)
    {
        $is_read = $request->get('is_read', 0);
        $messages = UserMessage::query()->where('is_read', $is_read)->where('admin_id', login_admin_id())->where('status', ModelStatusEnum::OK)->orderByDesc('id')->paginate();

        $messages = MessageResource::collection($messages);

        return Response::success($messages);
    }

    /**
     * 读一个消息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function readMessage(Request $request)
    {
        $this->validate($request, [
            'user_message_id' => 'required|integer'
        ]);

        $id = $request->get('user_message_id');
        $model = UserMessage::query()->where('admin_id', login_admin_id())->find($id);
        $model->is_read = 1;
        $model->save();
        return Response::success([]);
    }

    /**
     * 读全部消息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     */
    public function readMessages(Request $request)
    {
        UserMessage::query()->where('admin_id', login_admin_id())->where('is_read', 0)->update(['is_read' => 1]);
        return Response::success([]);
    }
}
