<?php

namespace App\Http\Controllers\Admin\School;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\PermissionEnum;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\School\LessonTeacher;
use App\Repositories\Models\School\Term;
use App\Repositories\Presenters\School\TermPresenter;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\School\TermRepository;
use App\Repositories\Validators\School\TermValidator;

/**
 * Class TermsController.
 *
 * @package namespace App\Http\Controllers\Mentor;
 */
class TermsController extends Controller
{
    /**
     * @var TermRepository
     */
    protected $repository;

    /**
     * @var TermValidator
     */
    protected $validator;

    /**
     * TermsController constructor.
     *
     * @param TermRepository $repository
     * @param TermValidator $validator
     */
    public function __construct(TermRepository $repository, TermValidator $validator)
    {
        parent::__construct();
        $this->repository = $repository;
        $this->validator = $validator;
    }

    /**
     * 列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        $lists = $this->repository->paginate($request->get('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }

    /**
     * 添加
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource|void
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function store(Request $request)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
        try {

            $end = Carbon::parse($data['end_date']);
            $start = Carbon::parse($data['start_date']);

            if ($start->dayOfWeek != 1) {
                return $this->error('开始日期必须为周一');
            }

            if ($end->dayOfWeek != 0) {
                return $this->error('结束日期必须为周日');
            }

            $weeks = $end->diffInWeeks($start);

            if ($data['weeks'] != $weeks) {
                return $this->error('周数不匹配');
            }

            $model = $this->repository->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->error($e);
        }
    }

    /**
     * 详情
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function show($id)
    {
        $data = $this->repository->find($id);
        return Response::success($data);
    }

    /**
     * 更新
     * @param Request $request
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function update(Request $request, $id)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));

            $end = Carbon::parse($data['end_date']);
            $start = Carbon::parse($data['start_date']);

            if ($start->dayOfWeek != 1) {
                return $this->error('开始日期必须为周一');
            }

            if ($end->dayOfWeek != 0) {
                return $this->error('结束日期必须为周日');
            }

            $weeks = $end->diffInWeeks($start);

            if ($data['weeks'] != $weeks) {
                return $this->error('周数不匹配');
            }
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->error($e);
        }
    }

    /**
     * 删除
     * @param $id
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function destroy($id)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        try {
            $this->repository->skipPresenter()->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 批量删除
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function delete(Request $request)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    /**
     * 选项
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function selectOptions(Request $request)
    {
        $data = $this->repository->skipPresenter()->all(['name', 'id', 'weeks']);
        return Response::success($data);
    }

    /**
     * 暂停
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     */
    public function pause(Request $request)
    {
        $this->isCan(PermissionEnum::controller_term_all);
        $this->validate($request, [
            'id' => 'required|integer',
            'type' => 'sometimes|nullable|integer',
        ]);
        $status = ModelStatusEnum::PAUSE;
        if ($request->get('type')) {
            $status = ModelStatusEnum::OK;
        }

        $re = Term::query()->where('id', $request->get('id'))->update(['status' => $status]);
        if ($re) {
            return Response::success([]);
        }
        return $this->error('操作失败');
    }
}
