<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Jobs\Lab;

use App\Jobs\Job;
use App\Repositories\Enums\Lab\ImportStatusEnum;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\School\TeachTypeEnum;
use App\Repositories\Models\Base\Admin;
use App\Repositories\Models\Lab\ImportRecord;
use App\Repositories\Models\School\Grade;
use App\Repositories\Models\School\Lesson;
use App\Repositories\Models\Lab\LessonSchedule;
use App\Repositories\Models\Lab\LessonTeacher;
use App\Repositories\Models\School\Room;
use App\Repositories\Models\School\Teacher;
use App\Repositories\Models\School\Term;
use App\Repositories\Models\School\Time;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class ImportLessonSchedulesJob extends Job
{

    private $recordModel;

    /**
     * Create a new job instance.
     */
    public function __construct($importRecordId)
    {
        $this->recordModel = ImportRecord::query()->find($importRecordId);
    }

    /**
     * 确定任务应该超时的时间
     *
     * @return \DateTime
     */
    public function retryUntil()
    {
        return Carbon::now()->addHours(8);
    }

    /**
     * Execute the job.
     */
    public function handle()
    {
        $model = $this->recordModel;

        LessonSchedule::query()->where('term_id', $model->term_id)->where('import_record_id', $model->id)->delete();

        switch ($model->type) {
            case 1:
                $this->load_template_1($model);
                break;
            case 2:
                $this->load_template_2($model);
                break;
        }
    }

    public function load_template_1($model)
    {
        $path = Storage::disk('public')->path($model->resources);

        $importRecord = ImportRecord::query()->find($model->id);
        $importRecord->status = ImportStatusEnum::IMPORT_IN;
        $importRecord->save();
        $term_id = $model->term_id;
        $start_day = Term::query()->where('id', $term_id)->value('start_date');

        if (!file_exists($path)) {
            $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
            $importRecord->message = '找不到文件';
            $importRecord->save();
            Log::error('*************找不到文件***************');
            Log::error($path);
            return false;
        }

        try {

            $config = ['path' => ''];
            $excel = new \Vtiful\Kernel\Excel($config);
            $excel->openFile($path)
                ->openSheet();
//            ->setSkipRows(1);

            //处理数据
            $data = [];
            $lessonRoom = [];
            $lessonHourRoom = 0;
            $testRoom = [];
            Log::error('*************开始读取数据*****************');
            while (($row = $excel->nextRow()) !== NULL) {
                $nums = count($row);
                if ($nums == 13) {
                    list(
                        $department,
                        $floor,
                        $room_no,
                        $room,
                        $lesson,
                        $lesson_hour,
                        $class_index,
                        $class_nums,
                        $test_name,
                        $test_class,
                        $test_student_nums,
                        $teacher,
                        $time
                        ) = $row;
                    Log::error("数据：" . php2js($row));
                    if (empty($room_no)) continue;
                    if ($department == '实验室所属部门') continue;

                    if (!empty($lesson)) {
                        $lesson = $this->decodeLesson($lesson);
                        $lessonRoom = $lesson;
                    }

                    if ($lesson_hour) {
                        $lessonHourRoom = $lesson_hour;
                    }

                    if ($test_name) {
                        $test_name = $this->decodeLesson($test_name);
                        $testRoom = $test_name;
                    }

                    $class = $this->decodeTestClass($test_class);

                    $teacher = $this->decodeLesson($teacher);

                    $da = $this->decodeTestTime($time);
                    $week = $this->getNum($da[2]);
                    $section = $da[1];
                    $week_index = $this->zh2int($da[0]);
                    $day = Carbon::parse($start_day)->addWeeks($week - 1)->addDays($week_index)->toDateString();

                    $data[] = [
                        'room' => $room,
                        'room_no' => $room_no,
                        'lesson' => $lessonRoom[1],
                        'lesson_no' => $lessonRoom[0],
                        'lesson_hour' => $lessonHourRoom,
                        'test' => $testRoom[1],
                        'test_no' => $testRoom[0],
                        'test_student_nums' => $test_student_nums,
                        'class' => $class[2],
                        'class_no' => $class[0],
                        'class_group' => ((int)($class[1])) - 1,
                        'class_student_nums' => $class_nums,
                        'teacher' => $teacher[1],
                        'teacher_no' => $teacher[0],
                        'section' => $section,
                        'week' => $week,
                        'week_index' => $week_index,
                        'day' => $day,
                        'remark' => '',
                    ];
                    $i = count($data);
                    Log::error("第{$i}条数据已完成：{$day}|{$section}|{$room}//{$lessonRoom[1]}-{$testRoom[1]}|{$class[2]}|{$teacher[1]}");
                } elseif ($nums == 12) {
                    $index = count($data) - 1;
                    $data[$index]['remark'] = $row[11];
                } else {
                    $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
                    $importRecord->message = '与模板文件不匹配';
                    $importRecord->save();
                    Log::error('*************与模板文件不匹配***************');
                    return true;
                }
            }
            Log::error("数据读取完成");
            $n = count($data);
            Log::error("开始写入数据：共{$n}条");
            //数据写入
            DB::beginTransaction();
            foreach ($data as $i => $item) {
                //教室
                $room_id = Room::query()->where('room_no', $item['room_no'])->value('id');
                if (empty($room_id)) {
                    $roomModel = Room::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['room'],
                        'room_no' => $item['room_no'],
                        'max_student_nums' => $item['class_student_nums'],
                        'status' => ModelStatusEnum::OK
                    ]);
                    $room_id = $roomModel->id;
                }

                //节次
                $time_ids = [];
                switch ($item['section']) {
                    case '1-2':
                    case '3-4':
                    case '5-6':
                    case '7-8':
                    case '9-11':
                        $time_id = Time::query()->where('section', $item['section'])->value('id');
                        if (empty($time_id)) {
                            $timeModel = Time::query()->create([
                                'name' => $item['section'],
                                'section' => $item['section'],
                                'status' => ModelStatusEnum::OK
                            ]);
                            $time_id = $timeModel->id;
                        }
                        $time_ids[] = $time_id;
                        break;
                    case '1-4':
                        $time_ids = Time::query()->whereIn('section', ['1-2', '3-4'])->pluck('id')->toArray();
                        break;
                    case '5-8':
                        $time_ids = Time::query()->whereIn('section', ['5-6', '7-8'])->pluck('id')->toArray();
                        break;
                }


                //老师
                $teacher_id = Teacher::query()->where('account', $item['teacher_no'])->value('id');
                if (empty($teacher_id)) {
                    $teacherModel = Teacher::query()->create([
                        'name' => $item['teacher'],
                        'account' => $item['teacher_no'],
                        'password' => $item['teacher_no'],
                        'status' => ModelStatusEnum::OK
                    ]);
                    $teacher_id = $teacherModel->id;
                }
                $teach_teacher_ids = [];
                $teacher_admin_id = Admin::query()->where('type', Teacher::class)->where('type_id', $teacher_id)->value('id');
                $teach_teacher_ids[] = $teacher_id;
                if (!empty($item['remark'])) {
                    $teacher = $this->decodeLesson($item['remark']);
                    $teacher_remark_id = Teacher::query()->where('account', $teacher['0'])->value('id');
                    if (empty($teacher_remark_id)) {
                        $teacherRemarkModel = Teacher::query()->create([
                            'name' => $teacher[1],
                            'account' => $teacher[0],
                            'password' => $teacher[0],
                            'status' => ModelStatusEnum::OK
                        ]);
                        $teacher_remark_id = $teacherRemarkModel->id;
                    }
                    $teach_teacher_ids[] = $teacher_remark_id;
                }
                //课程
                $lesson_id = Lesson::query()->where('no', $item['lesson_no'])->value('id');
                if (empty($lesson_id)) {
                    $lessonModel = Lesson::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['lesson'],
                        'no' => $item['lesson_no'],
                        'status' => ModelStatusEnum::OK
                    ]);
                    $lesson_id = $lessonModel->id;
                }
                //班级
                $grade_id = Grade::query()->where('name', $item['class'])->value('id');
                if (empty($grade_id)) {
                    $gradeModel = Grade::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['class'],
                        'no' => Str::random(8),
                        'status' => ModelStatusEnum::OK
                    ]);
                    $grade_id = $gradeModel->id;
                }

                $lessonTeacherModel = LessonTeacher::query()->firstOrCreate([
                    'import_record_id' => $importRecord->id,
                    'term_id' => $term_id,
                    'lesson_id' => $lesson_id,
                    'grade_id' => $grade_id,
                    'teacher_id' => $teacher_id,
                    'teacher_admin_id' => $teacher_admin_id,
                ]);

                foreach ($time_ids as $time_id) {
                    $is_clash = 0;
                    $where = [
                        'term_id' => $term_id,
                        'week' => $item['week'],
                        'week_index' => $item['week_index'],
                        'day' => $item['day'],
                        'time_id' => $time_id,
                        'room_id' => $room_id,
                        'status' => ModelStatusEnum::OK,
                    ];
                    if (\App\Repositories\Models\School\LessonSchedule::query()->where($where)->exists()) {
                        $is_clash = 1;
                    }

                    if (LessonSchedule::query()->where('import_record_id', $importRecord->id)->where($where)->exists()) {
                        $is_clash = 1;
                        LessonSchedule::query()->where($where)->update(['is_clash' => 1]);
                    }
                    LessonSchedule::query()->create([
                        'term_id' => $term_id,
                        'import_record_id' => $importRecord->id,
                        'lesson_teacher_id' => $lessonTeacherModel->id,

                        'week' => $item['week'],
                        'week_index' => $item['week_index'],
                        'day' => $item['day'],
                        'time_id' => $time_id,

                        'room_id' => $room_id,
                        'lesson_id' => $lesson_id,
                        'grade_id' => $grade_id,
                        'group' => $item['class_group'],
                        'student_nums' => $item['test_student_nums'],

                        'teach_type' => TeachTypeEnum::SJ,
                        'class_hour' => $item['lesson_hour'],
                        'teach_title' => $item['test'],
                        'teacher_id' => $teacher_id,
                        'teach_teacher_ids' => $teach_teacher_ids,
                        'teacher_admin_id' => $teacher_admin_id,
                        'is_clash' => $is_clash,
                        'status' => ModelStatusEnum::OK,
                    ]);
                }
                Log::error("正在创建：第{$i}条数据已完成");
            }

            DB::commit();
            Log::error("-----------完成---------------");
            $importRecord->status = ImportStatusEnum::IMPORT_OK;
            $importRecord->save();
        } catch (\Exception $exception) {
            $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
            $importRecord->message = $exception->getMessage();
            $importRecord->save();
            Log::error($exception);
            DB::rollBack();
        }
    }

    public function load_template_2($model)
    {
        try {
            $path = Storage::disk('public')->path($model->resources);

            $importRecord = ImportRecord::query()->find($model->id);
            $importRecord->status = ImportStatusEnum::IMPORT_IN;
            $importRecord->save();
            $term_id = $model->term_id;

            if (!file_exists($path)) {
                $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
                $importRecord->message = '找不到文件';
                $importRecord->save();
                Log::error('*************找不到文件***************');
                Log::error($path);
                return false;
            }

            $start_day = Term::query()->where('id', $term_id)->value('start_date');
            Log::error('*************开始读取数据**【模板二】***************');
            $config = ['path' => ''];
            $excel = new \Vtiful\Kernel\Excel($config);
            $excel->openFile($path)
                ->openSheet();

            //处理数据
            $data = [];
            while (($row = $excel->nextRow()) !== NULL) {
                if (count($row) != 12) {
                    $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
                    $importRecord->message = '与模板文件不匹配';
                    $importRecord->save();
                    Log::error('*************与模板文件不匹配***************');
                    return true;
                }
                list(
                    $week,
                    $week_index_name,
                    $section,
                    $room_name,
                    $lesson,
                    $teacher,
                    $grade_name,
                    $test_name,
                    $teach_name,
                    $lesson_hour,
                    $test_student_nums,
                    $group_name,
                    ) = $row;

                if ($week == '周次（必填）') continue;
                if (empty($week)) continue;

                $week_index = $this->zh2int($week_index_name);
                list($teacher_name, $teacher_no) = $this->decode2Teacher($teacher);

                list($room, $room_no) = $this->decode2Teacher($room_name);

                $day = Carbon::parse($start_day)->addWeeks($week - 1)->addDays($week_index)->toDateString();
                $group = $this->group2int($group_name);
                $data[] = [
                    'room' => $room,
                    'room_no' => $room_no,
                    'lesson' => $lesson,
                    'lesson_hour' => $lesson_hour,
                    'test' => $test_name,
                    'test_student_nums' => $test_student_nums,
                    'class' => $grade_name,
                    'class_group' => $group,
                    'teach_name' => $teach_name,
                    'teacher' => $teacher_name,
                    'teacher_no' => $teacher_no,
                    'section' => $section,
                    'week' => $week,
                    'week_index' => $week_index,
                    'day' => $day,
                    'remark' => '',
                ];
                $i = count($data);
                Log::error("第{$i}条数据已完成：{$day}|{$section}|{$room}//{$lesson}-{$test_name}|{$grade_name}|{$teacher_name}");
            }

            //数据写入
            Log::error("数据读取完成");
            $n = count($data);
            Log::error("开始写入数据：共{$n}条");
            DB::beginTransaction();
            foreach ($data as $i => $item) {
                Log::error("开始写入数据：第{$i}条");

                //教室
                $room_id = Room::query()->where('room_no', $item['room_no'])->value('id');
                Log::error("数据：" . php2js($item));
                if (empty($room_id)) {
                    $roomModel = Room::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['room'],
                        'room_no' => $item['room_no'],
                        'max_student_nums' => $item['test_student_nums'],
                        'status' => ModelStatusEnum::OK
                    ]);
                    $room_id = $roomModel->id;
                }
                //节次
                $time_ids = [];
                switch ($item['section']) {
                    case '1-2':
                    case '3-4':
                    case '5-6':
                    case '7-8':
                    case '9-11':
                        $time_id = Time::query()->where('section', $item['section'])->value('id');
                        if (empty($time_id)) {
                            $timeModel = Time::query()->create([
                                'name' => $item['section'],
                                'section' => $item['section'],
                                'status' => ModelStatusEnum::OK
                            ]);
                            $time_id = $timeModel->id;
                        }
                        $time_ids[] = $time_id;
                        break;
                    case '1-4':
                        $time_ids = Time::query()->whereIn('section', ['1-2', '3-4'])->pluck('id')->toArray();
                        break;
                    case '5-8':
                        $time_ids = Time::query()->whereIn('section', ['1-2', '5-8'])->pluck('id')->toArray();
                        break;
                }

                //老师
                $teacher_id = Teacher::query()->where('account', $item['teacher_no'])->value('id');
                if (empty($teacher_id)) {
                    $teacherModel = Teacher::query()->create([
                        'name' => $item['teacher'],
                        'account' => $item['teacher_no'],
                        'password' => $item['teacher_no'],
                        'status' => ModelStatusEnum::OK
                    ]);
                    $teacher_id = $teacherModel->id;
                }
                $teach_teacher_ids = [];
                $teacher_admin_id = Admin::query()->where('type', Teacher::class)->where('type_id', $teacher_id)->value('id');
                $teach_teacher_ids[] = $teacher_id;

                $lesson_id = Lesson::query()->where('name', $item['lesson'])->value('id');
                if (empty($lesson_id)) {
                    $lessonModel = Lesson::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['lesson'],
                        'no' => Str::random(8),
                        'status' => ModelStatusEnum::OK
                    ]);
                    $lesson_id = $lessonModel->id;
                }
                //班级
                $grade_id = Grade::query()->where('name', $item['class'])->value('id');
                if (empty($grade_id)) {
                    $gradeModel = Grade::query()->create([
                        'term_id' => $term_id,
                        'name' => $item['class'],
                        'no' => Str::random(8),
                        'status' => ModelStatusEnum::OK
                    ]);
                    $grade_id = $gradeModel->id;
                }

                $lessonTeacherModel = LessonTeacher::query()->firstOrCreate([
                    'import_record_id' => $importRecord->id,
                    'term_id' => $term_id,
                    'lesson_id' => $lesson_id,
                    'grade_id' => $grade_id,
                    'teacher_id' => $teacher_id,
                ]);

                foreach ($time_ids as $time_id) {
                    $is_clash = 0;
                    $where = [
                        'term_id' => $term_id,
                        'week' => $item['week'],
                        'week_index' => $item['week_index'],
                        'day' => $item['day'],
                        'time_id' => $time_id,
                        'room_id' => $room_id,
                        'status' => ModelStatusEnum::OK,
                    ];
                    if (\App\Repositories\Models\School\LessonSchedule::query()->where($where)->exists()) {
                        $is_clash = 1;
                    }
                    if (LessonSchedule::query()->where($where)->exists()) {
                        $is_clash = 1;
                        LessonSchedule::query()->where($where)->update(['is_clash' => 1]);
                    }

                    LessonSchedule::query()->create([
                        'term_id' => $term_id,
                        'lesson_teacher_id' => $lessonTeacherModel->id,
                        'import_record_id' => $importRecord->id,

                        'week' => $item['week'],
                        'week_index' => $item['week_index'],
                        'day' => $item['day'],
                        'time_id' => $time_id,

                        'room_id' => $room_id,
                        'lesson_id' => $lesson_id,
                        'grade_id' => $grade_id,
                        'group' => $item['class_group'],
                        'student_nums' => $item['test_student_nums'],

                        'teach_type' => TeachTypeEnum::SJ,
                        'class_hour' => $item['lesson_hour'],
                        'teach_title' => $item['test'],
                        'teacher_id' => $teacher_id,
                        'teach_teacher_ids' => $teach_teacher_ids,
                        'teacher_admin_id' => $teacher_admin_id,
                        'is_clash' => $is_clash,
                        'status' => ModelStatusEnum::OK,
                    ]);
                    Log::error("正在创建：第{$i}条数据已完成");
                }
            }
            DB::commit();
            Log::error("-----------完成---------------");
            $importRecord->status = ImportStatusEnum::IMPORT_OK;
            $importRecord->save();
        } catch (\Exception $exception) {
            $importRecord->status = ImportStatusEnum::IMPORT_ERROR;
            $importRecord->message = $exception;
            $importRecord->save();
            DB::rollBack();
        }
    }


    public function decode2Teacher($str)
    {
        return [
            $this->f1($str),
            $this->f4($str),
        ];
    }

    public function group2int($val)
    {
        return ord($val) - ord('A');
    }

    public function zh2int($str)
    {
        $map = [
            '一', '二', '三', '四', '五', '六', '七',
        ];
        return array_search($str, $map);
    }

    /**
     * 解码课程信息
     * @param $val
     * @return false|string[]
     */
    public function decodeLesson($val)
    {
        $val = trim(str_replace(PHP_EOL, '', $val));
        $arr = [
            $this->f4($val),
            str_replace('、', '', $this->f6($val)),
        ];
        return $arr;
    }

    public function f4($str)
    {
        $result = [];
        preg_match_all("/(?:\[)(.*)(?:\])/i", $str, $result);
        return $result[1][0];
    }

    public function f6($str)
    {
        $result = [];
        preg_match_all("/(?:\])(.*)/i", $str, $result);
        return $result[1][0];
    }

    /**
     * 处理实验班级
     * @param $val
     * @return false|string[]
     */
    public function decodeTestClass($val)
    {
        $val = trim(str_replace(PHP_EOL, '', $val));
        $arr = [
            $this->f1($val),
            $this->f4($val),
            $this->f3($val),
        ];
        return $arr;
    }

    public function f1($str)
    {
        $result = [];
        preg_match_all("/^(.*)(?:\[)/i", $str, $result);
        return $result[1][0];
    }

    public function f3($str)
    {
        $result = [];
        preg_match_all("/(?:\()(.*)(?:\))/i", $str, $result);
        return $result[1][0];
    }

    /**
     * 处理实验班级
     * @param $val
     * @return false|string[]
     */
    public function decodeTestTime($val)
    {
        $val = trim(str_replace(PHP_EOL, '', $val));
        $arr = [
            $this->f1($val),
            $this->f4($val),
            $this->f3($val),
        ];
        return $arr;
    }

    public function getNum($val)
    {
        preg_match('/\d+/', $val, $number);
        return (int)$number[0];
    }

    public function f2($str)
    {
        $result = [];
        preg_match_all("/(?:<)(.*)(?:>)/i", $str, $result);
        return $result[1][0];
    }

    public function f5($str)
    {
        $result = [];
        preg_match_all("/(?:\{)(.*)(?:\})/i", $str, $result);
        return $result[1][0];
    }
}
